/* -*- pse-c -*-
 *-----------------------------------------------------------------------------
 * Filename: micro_init_vlv.c
 *-----------------------------------------------------------------------------
 * Copyright (c) 2002-2013, Intel Corporation.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 *-----------------------------------------------------------------------------
 * Description:
 *  
 *-----------------------------------------------------------------------------
 */

#define MODULE_NAME hal.init

#include <io.h>
#include <pci.h>
#include <memmap.h>

#include <igd_core_structs.h>

#include <gn7/regs.h>
#include <gn7/context.h>

#include "../cmn/init_dispatch.h"

/*!
 * @addtogroup core_group
 * @{
 */

#ifdef CONFIG_VLV

extern unsigned char io_mapped;
extern unsigned short io_base;

extern int full_config_vlv(igd_context_t *context,
	init_dispatch_t *dispatch);
extern int get_revision_id_vlv(igd_context_t *context, os_pci_dev_t vga_dev);
extern void full_shutdown_vlv(igd_context_t *context);
static int query_vlv(igd_context_t *context,init_dispatch_t *dispatch,
	os_pci_dev_t vga_dev, unsigned int *bus, unsigned int *slot,
	unsigned int *func);
static int config_vlv(igd_context_t *context,
	init_dispatch_t *dispatch);
static int set_param_vlv(igd_context_t *context, unsigned long id,
	unsigned long value);
int get_param_vlv(igd_context_t *context, unsigned long id,
	unsigned long *value);
static void shutdown_vlv(igd_context_t *context);

extern os_pci_dev_t bridge_dev;

static platform_context_vlv_t platform_context_vlv;

init_dispatch_t init_dispatch_vlv = {
	"Intel(R) Atom Processor E3800 Product Family/ Intel(R) Celeron Processor N2920/J1900",
	"VLV_XXX",
	"hdmi",
	query_vlv,
	config_vlv,
	set_param_vlv,
	get_param_vlv,
	shutdown_vlv
};


/*!
 *
 * @param context
 * @param dispatch
 * @param vga_dev
 * @param bus
 * @param slot
 * @param func
 *
 * @return -IGD_ERROR_NODEV on failure
 * @return 0 on success
 */
static int query_vlv(
	igd_context_t *context,
	init_dispatch_t *dispatch,
	os_pci_dev_t vga_dev,
	unsigned int *bus,
	unsigned int *slot,
	unsigned int *func)
{
	platform_context_vlv_t *platform_context = &platform_context_vlv;

	uint32_t tmp;

	EMGD_TRACE_ENTER;

	/* So we don't have to pollute our function tables with multiple
	 * entries for every variant of VLV, update the device ID if one
	 * of the other SKUs is found
	 * FIXME - should we allow this, not yet knowing if
	 * we have codes that will need to differ for each SKU?
	 * --> context->device_context.did = PCI_DEVICE_ID_VGA_VLV;
	 */
	platform_context->igfx_dev.did = context->device_context.did;

	context->platform_context = (void *)&platform_context_vlv;

	/*
	 * Current specs indicate that Atom E6xx has only one PCI function.
	 * If this changes then we need to make sure we have func 0
	 * here as in previous chips.
	 */
	platform_context->igfx_dev.pcidev0 = vga_dev;

	/*
	 * finds the bus, device, func to be returned. Do this for D2:F0 only.
	 * the OS does not need to know the existence of D3:F0
	 */
	OS_PCI_GET_SLOT_ADDRESS(vga_dev, bus, slot, func);

	get_revision_id_vlv(context, vga_dev);
	platform_context->igfx_dev.rid = (unsigned char)context->device_context.rid;

	/*
	 * Lets get GMADR now.
	 * This must be in query so it is available early for the vBIOS.
	 * NOTE: VLV has GMMADR (map direct to dram- i.e. BSM)
	 */
	{

		if(OS_PCI_READ_CONFIG_32(vga_dev,
				VLV_PCI_GMADR_REG0, (void *)&context->device_context.fb_adr)) {
			EMGD_ERROR_EXIT("Reading BSM");
			return -IGD_ERROR_NODEV;
		}
		context->device_context.fb_adr &= 0xf8000000;
		if(OS_PCI_READ_CONFIG_32(vga_dev,
				VLV_PCI_GMADR_REG1, &tmp)) {
			EMGD_ERROR_EXIT("Reading BSM");
			return -IGD_ERROR_NODEV;
		}
		tmp &= 0x0000007f;
		context->device_context.fb_adr |= (((long long)tmp) << 32);
	}

	EMGD_DEBUG("BSM-or-GMADR = 0x%08llx", context->device_context.fb_adr);

	{
		/*
	 	* Read IO Base.
	 	* This must be in query so it is available early for the vBIOS.
	 	*/
		if(OS_PCI_READ_CONFIG_16(vga_dev, VLV_PCI_IOBAR, &io_base)) {
			EMGD_ERROR_EXIT("Reading IO Base");
			return -IGD_ERROR_NODEV;
		}

	}
	/* Base Address is defined in Bits 15:3*/
	io_base &= 0xfff8;
	EMGD_DEBUG("io @: 0x%x", io_base);

	/* Gen is always io_mapped */
	io_mapped = 1;

	EMGD_TRACE_EXIT;
	return 0;
}


/*!
 *
 * @param context
 * @param dispatch
 *
 * @return -IGD_ERROR_NODEV on failure
 * @return 0 on success
 */
static int config_vlv(igd_context_t *context,
	init_dispatch_t *dispatch)
{

	EMGD_TRACE_ENTER;

	OPT_MICRO_CALL(full_config_vlv(context, dispatch));

	EMGD_TRACE_EXIT;
	return 0;
}


/*!
 *
 * @param context
 * @param id
 * @param value
 *
 * @return -IGD_ERROR_INVAL on failure
 * @return 0 on success
 */
int get_param_vlv(igd_context_t *context, unsigned long id,
	unsigned long *value)
{
	int ret = 0;
	unsigned long control_reg;
	unsigned char * mmio = context->device_context.virt_mmadr;

	EMGD_TRACE_ENTER;

	EMGD_DEBUG("ID: 0x%lx", id);

	/* Scratch registers used as below:
	 *
	 * 0x71410:
	 * --------
	 *   Bits 31-16 - EID Firmware identifier 0xE1DF
	 *   Bits 15-00 - Tell what data is present.
	 * Here are bits for what we are using know:
	 *   Bit 0 - Panel id
	 *   Bit 1 - List of ports for which displays are attached
	 *   Bit 2 - Memory reservation
	 * If any of the above bits is set that mean data is followed
	 * in the next registers.
	 *
	 * 0x71414:
	 * --------
	 *   Bits 07-00 - Panel Id
	 *   Bits 11-08 - Port list
	 * Information for Port list: If any of the bit is set means,
	 *   a display is attached to that port as follows:
	 *    Bit 08 - CRT
	 *    Bit 09 - Internal LVDS
	 *    Bit 10 - SDVOB/HDMI-DVI-B/DP-B
	 *    Bit 11 - HDMI-DVI-C/DP-C
	 *
	 * 0x71418:
	 * --------
	 * Bits 15-00 - Reserved Memory value in number of 4k size pages
	 */
	*value = 0;
	control_reg = READ_MMIO_REG_VLV(mmio, 0x71410);
	*value = 0;

	switch(id) {
		case IGD_PARAM_PORT_LIST:
			/*
			 * Verify that the Embedded Firware is present.
			 */
			if ((control_reg>>16) != 0xE1DF) {
				EMGD_DEBUG("Exit No Embedded vBIOS found");
				EMGD_TRACE_EXIT;
				return -IGD_ERROR_INVAL;
			}

			/*
			 * If the port list bit is set in control register,
			 * read the port list
			 */
			if (control_reg & 0x2) {
				unsigned char temp;
				int i = 0;

				temp = (unsigned char)((READ_MMIO_REG_VLV(mmio, 0x71414)>>8) & 0xFF);
				EMGD_DEBUG("Connected Port Bits: 0x%x", temp);

				/*
				 * The meanings of bits in temp were dictated by VBIOS
				 * and should not change due to backward compatibility
				 * with Legacy VBIOS
				 */
				if (temp & 0x02) {
					/* Internal LVDS port */
					value[i++] = 4;
				}
				if (temp & 0x04) {
					/* SDVOB or HDMI-DVI-B Port */
					value[i++] = 2;
				}
				/* FIXME - do we need to differnentiate DP from HDMI?
				 * FIXME - what about MIPI and emb-DP?
				 *       - Dont have numbers assigned for these yet
				 */
			} else {
				EMGD_DEBUG("Port List read failed: Incorrect Operation");
				ret = -IGD_ERROR_INVAL;
			}
			break;

		case IGD_PARAM_PANEL_ID:
			/*
			 * Check for Embedded firmware
			 */
			if ((control_reg>>16) != 0xE1DF) {
				EMGD_DEBUG("No Embedded vBIOS found");
				EMGD_TRACE_EXIT;
				return -IGD_ERROR_INVAL;
			}

			/*
			 * The panel id bit must be set in the control register
			 * to indicate valid panel (config) ID value.
			 */
			if (control_reg & 0x1) {
				*value = READ_MMIO_REG_VLV(mmio, 0x71414) & 0xFF;
				if(!(*value)) {
					/* we cannot allow for config id = 0 */
					ret = -IGD_ERROR_INVAL;
				}
			} else {
				EMGD_DEBUG("Panel ID read failed: Incorrect Operation");
				ret = -IGD_ERROR_INVAL;
			}
			break;
		case IGD_PARAM_MEM_RESERVATION:
			/*
			 * Check for Embedded firmware
			 */
			if ((control_reg>>16) != 0xE1DF) {
				EMGD_DEBUG("No Embedded vBIOS found");
				EMGD_TRACE_EXIT;
				return -IGD_ERROR_INVAL;
			}

			/*
			 * The mem reservation bit must be set in the control register
			 * to indicate valid mem reservation value.
			 */
			if (control_reg & 0x4) {
				*value = (READ_MMIO_REG_VLV(mmio, 0x71418) & 0xFFFF);
			} else {
				EMGD_DEBUG("Mem Reservation read failed: Incorrect Operation");
				ret = -IGD_ERROR_INVAL;
			}
			break;
		default:
			ret = -IGD_ERROR_INVAL;
			break;
	}

	EMGD_TRACE_EXIT;
	return ret;
}

/*!
 *
 * @param context
 * @param id
 * @param value
 *
 * @return -IGD_ERROR_INVAL
 */
static int set_param_vlv(igd_context_t *context, unsigned long id,
	unsigned long value)
{
	EMGD_ERROR("FIXME: set_param_vlv not implemented!!");
	return 0;
}

/*!
 *
 * @param context
 *
 * @return void
 */
static void shutdown_vlv(igd_context_t *context)
{

	OPT_MICRO_VOID_CALL(full_shutdown_vlv(context));
}


/*!
 *
 * @param context
 * @param dispatch
 * @param vga_dev
 *
 * @return -IGD_ERROR_NODEV on failure
 * @return 0 on success
 */
int get_revision_id_vlv(igd_context_t *context,
	os_pci_dev_t vga_dev)
{
	platform_context_vlv_t *platform_context;

	EMGD_TRACE_ENTER;
  	 
	platform_context = (platform_context_vlv_t *)context->platform_context;

	/* Read RID */
	if(OS_PCI_READ_CONFIG_8(vga_dev, PCI_RID,
		(unsigned char *)&context->device_context.rid)) {
		EMGD_ERROR_EXIT("Error occured reading RID");
		return -IGD_ERROR_NODEV;
	}
  	 
	EMGD_DEBUG(" rid = 0x%lx", context->device_context.rid);
	
	EMGD_TRACE_EXIT;
	return 0;
}


/*!
 * Program the GTT.
 *
 * @param context
 *
 * @return gtt_init_ret_code
 * @return 0
 */
void gtt_init_vlv(igd_context_t *context)
{
	/*
	 * Program the GTT.
	 * The GTT is programmed as follows when NOT using A000 for storing
	 * Dynamic Data:
	 *                    8 Generic case             512MB RAM with
	 *                                            256MB Aperture and
	 *                                            32MB Stolen memory
	 *                                                 Example
	 *                 --------------------     --------------------
	 *     ------------Top of RAM (TR)                512MB
	 *     |        |  (TR)
	 *     | Stolen |
	 *     |        |  (DOES NOT include GTT)
	 * S M | Video  |
	 *     |        |  TR-32MB (PCI Cfg = GMS)        Base = 512MB - (32MB - 4K)
	 * t e | Mem    |                                 32MB size depends on GMS
	 *     |        |                                 of GGC (32 to 512M)
	 *     |        |                                 (Also Base = BDSM + 4K)
	 * o m | - - - -|  4KB scratch page
	 *     |(Scrtch)|  (part of stolen vid mem)       Base = 512MB - 32MB
	 * l o |        |                                 (Also Base = BDSM)
	 *     |--------|--Base of Data Stolen Memory
	 * e r |        |  (PCI Cfg = BDSM)
	 *     |  GTT   |
	 * n y |  Mem   |  Global GTT Memory              Base = 512MB - (32B + 2MB)
	 *     |        |  (access via GTT(MM)ADR         (2MB or 1MB depending
	 *     |        |                                 on GGMS of GGC)
	 *     |--------|--Base of GTT Stolen Memory      (Also Base = BGSM)
	 *     |        |  (PCI Cfg = BGSM)
	 *     | System |
	 *     |  Mem   |
	 *     |        |
	 *     ----------  00000000                       0
	 *
	 *
	 * The GTT is programmed as follows when using A000 for storing
	 * Dynamic Data:
	 *                     Generic case             512MB RAM with
	 *                                            256MB Aperture and
	 *                                            32MB Stolen memory
	 *                                                 Example
	 *                 --------------------     --------------------
	 *     ------------Top of RAM (TR)                512MB
	 *     |        |  (TR)
	 *     | Stolen |
	 *     |        |  (DOES NOT include GTT)
	 * S M | Video  |  32MB (PCI Cfg = GMS)           Base = 512MB -
	 *     |        |                                  (32MB-64K)
	 * t e | Mem    |                                  *Also = BDSM + 64K
	 *     | - - - -|
	 * o m |(Scrtch)|  4KB Scratch Page               Base = 512MB -
	 *     |        |  (part of 32MB stolen vid)       (32MB-4K)
	 * l o |- - - - |                                  *Also = BDSM + 60K
	 *     |Dyn-A000|  60K Dynamic A000 Data          Base = 512MB - 32MB
	 * e r |        |  (part of 32MB stolen vid )      *Also = BDSM
	 *     |--------|--Base of Data Stolen Memory     512-32MB (32MB size
	 * n y |        |  (PCI Cfg = BDSM)               depends on GMS of GGC)
	 *     |  GTT   |
	 *     |  Mem   |  Global GTT Memory              2MB (or 1MB depending
	 *     |        |  (access via GTT(MM)ADR         on GGMS of GGC)
	 *     |        |
	 *     |--------|--Base of GTT Stolen Memory      512-32MB-2MB
	 *     |        |  (PCI Cfg = BGSM)
	 *     | System |
	 *     |  Mem   |
	 *     |        |
	 *     ----------  00000000                       0
	 *
	 */

	int gtt_init_ret_code;
	os_pci_dev_t vga_dev;
	platform_context_vlv_t *platform_context;
	long long phys_stolen_mem, page_table_entry;
	uint32_t pci_read_long;
	unsigned char *mmio;
	unsigned long *virt_gttadr;
	struct page *scratch_page;
	long long scratch_physaddr;

	EMGD_TRACE_ENTER;

	platform_context = (platform_context_vlv_t *)context->platform_context;
	vga_dev = platform_context->igfx_dev.pcidev0;
	mmio = context->device_context.virt_mmadr;
	virt_gttadr = context->device_context.virt_gttadr;

	/* Enable the PCI Device */
	gtt_init_ret_code = OS_ENABLE_PCI(vga_dev);
	if(gtt_init_ret_code) {
		EMGD_ERROR("OS_ENABLE_PCI failed")
		EMGD_TRACE_EXIT;
		return;
	}

	/* Create a scratch page using alloc_page for compatibility with Koheo */
	scratch_page = alloc_page(GFP_DMA32 | __GFP_ZERO);
	context->device_context.scratch_page = scratch_page;

	/* In VLV, GTT is not at above graphics stolen memory,
	 * stolen_pages = reserved_mem found earlier
	 * stolen_pages is initialized here for compatibility with Koheo */
	context->device_context.stolen_pages = context->device_context.reserved_mem;

	/* Get Stolen Memory Base Address */
	OS_PCI_READ_CONFIG_32(vga_dev, VLV_PCI_BDSM_REG, &pci_read_long);
	context->device_context.reserved_mem_base_addr =
		pci_read_long & 0xFFF00000;

	EMGD_DEBUG("PCI_BDSM_REG = %x", pci_read_long);

	/* Physical address of the beginning of video memory */
	phys_stolen_mem = context->device_context.reserved_mem_base_addr;


	/* FIXME - Temporary VLV-A0 workaround - when we move to GEM 3.6, 
     * and we disconnect from Agpgart, then we can remove this
     * along with all the core-init code that touches the GTT
     * FYI - this code works fine on VLV-X0.
     */
	WRITE_MMIO_REG_VLV(phys_stolen_mem + (6*1024*1024), mmio, 0x182120);

	/* In VLV (like Gen7) PGTBL_CTL is not used, physical GTT location is hardwired in HW
	 * and is based on the BGSM register that is allocated by BIOS */

	WRITE_MMIO_REG_VLV(context->device_context.reserved_mem, mmio, 0x71418);
		/* write the num of stolen page (reserved_mem) into the scratch register SWF12 */

	/* Page table entry for scratch page */
	scratch_physaddr = page_to_phys(scratch_page) | 1;
	EMGD_DEBUG("Scratch page phys addr = 0x%llx",
			(unsigned long long)page_to_phys(scratch_page));

#define FIRMWARE_COMPLIANT_GTT_PROGRAMMING 1
#ifdef FIRMWARE_COMPLIANT_GTT_PROGRAMMING
	/* Program the GTT */
	//phys_stolen_mem += 4096;
		/*FIXME: Should we assume we cant touch the first
		 * 4K of the BDSM as firmware's scratch???!!!!*/

	for(page_table_entry = 0;
		page_table_entry < context->device_context.reserved_mem;
		page_table_entry++) {
		/* Enable */
		EMGD_WRITE32((phys_stolen_mem | 1),
				virt_gttadr + page_table_entry);
		phys_stolen_mem += 4096;
	}

	/* Program the remainder of the GTT to the scratch page. */
	for(page_table_entry = context->device_context.reserved_mem;
		page_table_entry < (context->device_context.gatt_pages);
		page_table_entry++) {
		/* Enable */
		EMGD_WRITE32(scratch_physaddr, virt_gttadr + page_table_entry);
	}
#else
	/* For VLV debug purpose, program the whole GTT with scratch page */
	for(page_table_entry = 0;
		page_table_entry < (context->device_context.gatt_pages);
		page_table_entry++) {
		/* Enable */
		EMGD_WRITE32(scratch_physaddr, virt_gttadr + page_table_entry);
	}
#endif
	EMGD_TRACE_EXIT;
	return;
}
//#endif

/* gtt_init_vlv_64bitcompliant??
 *
 * Using the Linux PCI commands
 * should be the way to go for
 * 32-bit vs 64-bit addressing???
 *
 * TAKE NOTE: FOLLOWING CODE from TunnelCreek as
 * reference so we need to review and replace for VLV
 *
 */
#if 0
/*
 * Initialize the GTT.
 *   - Find the size of stolen memory
 *   - Add stolen memory to the GTT
 *   - Map the GTT and video memory
 */
static void gtt_init_vlv(igd_context_t *context)
{
	struct drm_device *dev;
	unsigned char *mmio = context->device_context.virt_mmadr;
	unsigned long dvmt_mode = 0;
	unsigned long gtt_pages = 0;
	unsigned long stolen_mem_size = 0;
	unsigned long scratch;
	unsigned long base;
	unsigned long pte;
	unsigned short gmch_ctl;
	unsigned long pge_ctl;
	unsigned long gtt_phys_start;
	unsigned long gatt_start;
	unsigned long gatt_pages;
	unsigned long gtt_start;
	unsigned long gtt_order;
	unsigned long stolen_mem_base;
	unsigned long *gtt_table;
	struct page *gtt_table_page;
	int i;

	dev = (struct drm_device *)context->drm_dev;

	/* Enable the GMCH */
	OS_PCI_READ_CONFIG_16((os_pci_dev_t)dev->pdev, PSB_GMCH_CTRL,
			&gmch_ctl);
	OS_PCI_WRITE_CONFIG_16((os_pci_dev_t)dev->pdev, PSB_GMCH_CTRL,
			(gmch_ctl | PSB_GMCH_ENABLED));
	context->device_context.gmch_ctl = gmch_ctl;

	/* Get the page table control register */
	pge_ctl = readl(mmio + PSB_PGETBL_CTL);
	gtt_phys_start = pge_ctl & PAGE_MASK;

	/* Create a scratch page to initialize empty GTT entries */
	context->device_context.scratch_page = alloc_page(GFP_DMA32 | __GFP_ZERO);

	/*
	* Is pci_resource_start(dev->pdev, PSB_GATT_RESOURCE); the same
	* as pci_read_config_dword(dev->pdev, 0x1C, &value)?
	*
	* PSB_GATT_RESOURCE length is the amount of memory addressable
	* by the GTT table.
	*/
	gatt_start = pci_resource_start(dev->pdev, PSB_GATT_RESOURCE);
	gatt_pages = (pci_resource_len(dev->pdev, PSB_GATT_RESOURCE) >> PAGE_SHIFT);
	context->device_context.gatt_pages = gatt_pages;

	/*
	 * The GTT wasn't set up by the vBios
	 */
	if (1){
		/* (!pge_ctl) - always yes for VLV? no PGTBL + base addreses hardwired?*/
		context->device_context.stolen_pages = 0;

		gtt_pages = pci_resource_len(dev->pdev, PSB_GTT_RESOURCE) >> PAGE_SHIFT;
		gtt_order = get_order(gtt_pages << PAGE_SHIFT);
		gtt_table = (unsigned long *)__get_free_pages(GFP_KERNEL, gtt_order);
		/* Make sure allocation was successful */
		if (NULL == gtt_table) {
			EMGD_ERROR("Failed to allocate kernel pages for GTT");
			return;
		}
		context->device_context.virt_gttadr = gtt_table;

		for (i=0; i < (1 << gtt_order); i++) {
			gtt_table_page = virt_to_page(gtt_table + (PAGE_SIZE * i));
			EMGD_DEBUG("Setting reserved bit on %p", gtt_table_page);
			set_bit(PG_reserved, &gtt_table_page->flags);
		}

		gtt_phys_start = virt_to_phys(gtt_table);

		for (i = 0; i < gtt_pages; i++) {
			gtt_table[i] = (unsigned long)context->device_context.scratch_page;
		}

		EMGD_DEBUG("Detected GTT was not enabled by firmware");
		EMGD_DEBUG("GMMADR(region 0) start: 0x%08lx (%ldM).",
			gatt_start, (gatt_pages / 256));
		EMGD_DEBUG("GTTADR(region 3) start: 0x%08lx (can map %ldM RAM), and "
			"actual RAM base 0x%08lx.",
			(unsigned long)gtt_table, (gtt_pages * 4), gtt_phys_start);

		/* Enable the newly created GTT */
		EMGD_DEBUG("Enabling new GTT");
		writel(gtt_phys_start, mmio + PSB_PGETBL_CTL);
		pge_ctl = readl(mmio + PSB_PGETBL_CTL);

	}

	/* Update the scratch registers to say we have no stolen memory */
	scratch = readl(mmio + SCR1);
	if ((scratch & FW_ID) == FW_ID) {
		/* if an EMGD vBios modify only the stolen memory bit */
		scratch |= ST_BIT;
		writel(scratch, mmio + SCR1);
	} else {
		/* Not an EMGD vBios so just set the entire register to a known value */
		writel((FW_ID|ST_BIT), mmio + SCR1);
	}

	/*
	 * Report back that there is 0MB of stolen memory regardless of
	 * what was really in there.  Fresh pages will be inserted over
	 * the top of the existing stolen memory.
	 */
	writel(0, mmio + SCR2);

	/*
	 * FIXME: Shouldn't this fill in all the GTT page table entries with
	 * the scratch page?
	 */

	return;
}
#endif

#endif
