/* -*- pse-c -*-
 *-----------------------------------------------------------------------------
 * Filename: micro_ovl.c
 *-----------------------------------------------------------------------------
 * Copyright (c) 2002-2013, Intel Corporation.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 *-----------------------------------------------------------------------------
 * Description:
 *  
 *-----------------------------------------------------------------------------
 */

#define MODULE_NAME hal.overlay

#include <context.h>
#include <memory.h>
#include <dsp.h>
#include <io.h>

#include <igd_mode.h>
//#include <igd_ovl.h>
#include <igd_pwr.h>

#include <general.h>
#include <mode_access.h>
#include <dispatch.h>
#include <intelpci.h>

#include "ovl_dispatch.h"
#include "ovl_virt.h"

/*
 * TODO: If there is micro overlay for device dependent layer
 * modify extern below to the correct micro device dispatch
 * Only two micro overlay existing is for Plb and Ctg
 */

extern ovl_dispatch_t ovl_micro_dispatch_snb[];
extern ovl_dispatch_t ovl_micro_dispatch_vlv[];

static dispatch_table_t ovl_micro_dispatch_list[] = {
/*
 * TODO: Fix this.
 *  * This table would point the micro dispatch tables.
 *  For now, other than Plb ,refer the micro dispatch
 *  to the original table
 * */

#ifdef CONFIG_SNB
	{PCI_DEVICE_ID_VGA_SNB, &ovl_micro_dispatch_snb},
#endif
#ifdef CONFIG_IVB
	{PCI_DEVICE_ID_VGA_IVB, &ovl_micro_dispatch_vlv},
#endif
#ifdef CONFIG_VLV
	{PCI_DEVICE_ID_VGA_VLV, &ovl_micro_dispatch_vlv},
	{PCI_DEVICE_ID_VGA_VLV2, &ovl_micro_dispatch_vlv},
	{PCI_DEVICE_ID_VGA_VLV3, &ovl_micro_dispatch_vlv},
	{PCI_DEVICE_ID_VGA_VLV4, &ovl_micro_dispatch_vlv},
#endif
	{0, NULL}
};
extern int ovl_full_init(igd_context_t *context, igd_param_t *params,ovl_context_t *ovl_context);

#ifndef CONFIG_MICRO_OVERLAY
#define OVL_FULL_INIT(a, b, c) ovl_full_init(a, b, c)
#else
#define OVL_FULL_INIT(a, b, c) 1
#endif

extern ovl_context_t ovl_context[1];
static int igd_micro_alter_ovl(igd_display_h display_h,
	igd_appcontext_h     appctx,
	emgd_framebuffer_t  *src_surf,
	igd_rect_t          *src_rect,
	igd_rect_t          *dest_rect,
	igd_ovl_info_t      *ovl_info,
	unsigned int         flags)
{
	igd_display_context_t *display =
		(igd_display_context_t *)display_h;
	ovl_dispatch_t    *ovl_dispatch =
		(ovl_dispatch_t *)ovl_context->dispatch;

	igd_display_context_t *ovl_display;

	int ret = 0;

	EMGD_TRACE_ENTER;

	ovl_display = display;

	/* Call family dependent overlay function to alter the overlay. */
	ret = ovl_dispatch->alter_ovl(ovl_display, appctx,
				src_surf, src_rect,
				dest_rect, ovl_info, flags);

	if (ret != IGD_SUCCESS) {
		EMGD_ERROR("Error micro alter overlay");
		/* Turn the overlay off when there is an error */
		ovl_dispatch->alter_ovl(ovl_display, NULL,
					NULL, NULL, NULL, NULL, IGD_OVL_ALTER_OFF);
	}

	EMGD_TRACE_EXIT;
	return ret;
}

int _overlay_init(igd_context_t *context, igd_param_t *params)
{

	EMGD_TRACE_ENTER;

	if (OVL_FULL_INIT(context, params, ovl_context)) {
		/*
		 * Even if full init failed we can still use the micro-gmm
		 */
		EMGD_DEBUG("Full overlay did not initialize, using micro-overlay");

	}

	EMGD_TRACE_EXIT;
	return IGD_SUCCESS;
}
